@@ -2,8 +2,19 @@
 # -*- coding: UTF-8 -*-
 import dataclasses
 import enum
-from typing import List, Tuple, Union, NewType, Any, Dict, Mapping, Collection, Hashable, Type, \
-    NamedTuple
+from typing import (
+    List,
+    Tuple,
+    Union,
+    NewType,
+    Any,
+    Dict,
+    Mapping,
+    Collection,
+    Hashable,
+    Type,
+    NamedTuple,
+)
 from collections import UserList
 
 from .util import DictFactory, isnamedtuple, Nothing
@@ -11,72 +22,81 @@ from .util import DictFactory, isnamedtuple, Nothing
 
 class MathOps(str, enum.Enum):
     """Common SQL arithmetic operations."""
-    ADD = '+'
-    IADD = '+='
-    SUB = '-'
-    ISUB = '-='
-    MUL = '*'
-    IMUL = '*='
-    DIV = '/'
-    IDIV = '/='
-    MOD = '%'
-    IMOD = '%='
+
+    ADD = "+"
+    IADD = "+="
+    SUB = "-"
+    ISUB = "-="
+    MUL = "*"
+    IMUL = "*="
+    DIV = "/"
+    IDIV = "/="
+    MOD = "%"
+    IMOD = "%="
 
 
 class BitOps(str, enum.Enum):
     """SQL Bitwise operations."""
-    AND = '&'
-    IAND = '&='
-    OR = '|'
-    IOR = '|*='
-    XOR = '^'
-    IXOR = '^-='
+
+    AND = "&"
+    IAND = "&="
+    OR = "|"
+    IOR = "|*="
+    XOR = "^"
+    IXOR = "^-="
 
 
 class LogOps(str, enum.Enum):
     """Common SQL Logical Operations."""
-    ALL = 'ALL'
-    AND = 'AND'
-    ANY = 'ANY'
-    BET = 'BETWEEN'
-    EX = 'EXISTS'
-    IN = 'IN'
-    ILI = 'ILIKE'
-    LI = 'LIKE'
-    NOT = 'NOT'
-    OR = 'OR'
-    RE = 'REGEXP'
+
+    ALL = "ALL"
+    AND = "AND"
+    ANY = "ANY"
+    BET = "BETWEEN"
+    EX = "EXISTS"
+    IN = "IN"
+    ILI = "ILIKE"
+    LI = "LIKE"
+    NOT = "NOT"
+    OR = "OR"
+    RE = "REGEXP"
 
 
 class CmpOps(str, enum.Enum):
     """Common SQL Comparison operations."""
-    EQ = '='
-    GT = '>'
-    GE = '>='
-    LT = '<'
-    LE = '<='
-    NE = '<>'
+
+    EQ = "="
+    GT = ">"
+    GE = ">="
+    LT = "<"
+    LE = "<="
+    NE = "<>"
 
 
 class BasicParamStyle(str, enum.Enum):
     """Simple DBAPI 2.0 compliant param styles."""
-    QM = '?'
-    FM = '%s'
+
+    QM = "?"
+    FM = "%s"
 
 
 class NumParamStyle(str, enum.Enum):
     """Numbered DBAPI 2.0 compliant param styles."""
-    NUM = ':{}'
-    DOL = '${}'
+
+    NUM = ":{}"
+    DOL = "${}"
 
 
 class NameParamStyle(str, enum.Enum):
     """Named DBAPI 2.0 compliant param styles"""
-    NAME = ':{}'
-    PYFM = '%({})s'
+
+    NAME = ":{}"
+    PYFM = "%({})s"
 
 
-ParamStyleType = NewType('ParamStyleType', Union[BasicParamStyle, NumParamStyle, NameParamStyle])
+ParamStyleType = NewType(
+    "ParamStyleType", Union[BasicParamStyle, NumParamStyle, NameParamStyle]
+)
 DEFAULT_PARAM_STYLE = NumParamStyle.NUM
 
 
@@ -87,19 +107,25 @@ class Field:
     A ``Field`` represents the most basic component of a SQL statement. It can be used for identifying
     the fields which you plan to select or modify or the fields which you wish to filter by.
     """
+
     name: str = None
     value: Any = None
 
     def __post_init__(self):
         try:
-            assert self.name or self.value, \
-                f'{type(self).__name__}.name or {type(self).__name__}.value must be provided'
+            assert (
+                self.name or self.value
+            ), f"{type(self).__name__}.name or {type(self).__name__}.value must be provided"
         except AssertionError as err:
             raise TypeError(err)
 
     def for_fetch(self) -> str:
         """Generate valid SQL for a ``Field`` if it is being used in a SELECT or RETURNING statement."""
-        return f"{self.name} AS {self.value}" if (self.value and self.name) else (self.name or self.value)
+        return (
+            f"{self.name} AS {self.value}"
+            if (self.value and self.name)
+            else (self.name or self.value)
+        )
 
 
 @dataclasses.dataclass
@@ -111,19 +137,22 @@ class Filter:
         - The ``Filter.opcode`` provides the operation (equal-to, less-than, ...).
         - The ``Filter.prefix``  an optional prefix to provide to the parameter naming.
     """
+
     field: Field
     opcode: CmpOps = CmpOps.EQ
-    prefix: str = ''
+    prefix: str = ""
 
     def __post_init__(self):
         try:
-            assert self.field.name and self.field.value is not None, \
-                f'{type(self).__name__}.field must have name and value.'
+            assert (
+                self.field.name and self.field.value is not None
+            ), f"{type(self).__name__}.field must have name and value."
         except AssertionError as err:
             raise TypeError(err)
 
-    def to_sql(self, args: 'ArgList' = None, style: ParamStyleType = DEFAULT_PARAM_STYLE) -> \
-            Tuple[str, 'ArgList']:
+    def to_sql(
+        self, args: "ArgList" = None, style: ParamStyleType = DEFAULT_PARAM_STYLE
+    ) -> Tuple[str, "ArgList"]:
         """Generate a single filter clause of a SQL Statement.
 
         Parameters
@@ -153,6 +182,7 @@ class FieldList(UserList):
 
     A custom list implementation for housing :class:`Field`.
     """
+
     def __init__(self, initlist: Collection[Field] = None):
         """The Constructor.
 
@@ -202,7 +232,10 @@ class ArgList(FieldList):
     Also provides a convenience method for outputting your args in the appropriate format
     for the selected style
     """
-    def for_sql(self, style: ParamStyleType = DEFAULT_PARAM_STYLE) -> Union[Dict[str, Any], List[Any]]:
+
+    def for_sql(
+        self, style: ParamStyleType = DEFAULT_PARAM_STYLE
+    ) -> Union[Dict[str, Any], List[Any]]:
         """Output the list of args in the appropriate format for the param-style.
 
         Parameters
@@ -220,6 +253,7 @@ class FilterList(UserList):
 
     A custom list implementation for housing :class:`Filter`.
     """
+
     def __init__(self, initlist: Collection[Filter] = None):
         initlist = initlist or []
         super().__init__(initlist)
@@ -239,8 +273,9 @@ class FilterList(UserList):
             raise TypeError(f"{type(self).__name__} requires type Filter.")
         super().append(item)
 
-    def to_sql(self, args: ArgList = None, style: ParamStyleType = DEFAULT_PARAM_STYLE) -> \
-            Tuple[str, ArgList]:
+    def to_sql(
+        self, args: ArgList = None, style: ParamStyleType = DEFAULT_PARAM_STYLE
+    ) -> Tuple[str, ArgList]:
         """Generate the ``WHERE`` clause of a SQL statement.
 
         Parameters
@@ -259,9 +294,9 @@ class FilterList(UserList):
         for fylter in self:
             sql, args = fylter.to_sql(args, style)
             where.append(sql)
-        where = 'AND\n  '.join(where)
+        where = "AND\n  ".join(where)
 
-        return f"WHERE\n  {where}" if where else '', args
+        return f"WHERE\n  {where}" if where else "", args
 
 
 @dataclasses.dataclass
@@ -269,9 +304,9 @@ class BaseSQLStatement:
     """A Base-class for simple SQL Statements (Select, Update, Insert, etc)"""
 
     def __post_init__(self):
-        if hasattr(self, 'fields') and not isinstance(self.fields, FieldList):
+        if hasattr(self, "fields") and not isinstance(self.fields, FieldList):
             self.fields = FieldList(self.fields)
-        if hasattr(self, 'filters') and not isinstance(self.filters, FilterList):
+        if hasattr(self, "filters") and not isinstance(self.filters, FilterList):
             self.filters = FilterList(self.filters)
 
     @property
@@ -288,6 +323,7 @@ class Select(BaseSQLStatement):
 
     As the first line indicates, this does not currently support JOINs.
     """
+
     table: str
     schema: str = None
     filters: FilterList = dataclasses.field(default_factory=FilterList)
@@ -301,11 +337,13 @@ class Select(BaseSQLStatement):
         columns = []
         for field in self.fields:
             columns.append(field.for_fetch())
-        columns = ',\n  '.join(columns) if columns else '*'
+        columns = ",\n  ".join(columns) if columns else "*"
 
         return f"SELECT\n  {columns}\nFROM\n  {self.table_name}"
 
-    def to_sql(self, style: ParamStyleType = DEFAULT_PARAM_STYLE) -> Tuple[str, Union[List, Dict]]:
+    def to_sql(
+        self, style: ParamStyleType = DEFAULT_PARAM_STYLE
+    ) -> Tuple[str, Union[List, Dict]]:
         """Generate a valid SQL SELECT statement for a single table.
 
         Parameters
@@ -324,7 +362,6 @@ class Select(BaseSQLStatement):
 
 
 class _BaseWriteStatement(BaseSQLStatement):
-
     def get_returning(self) -> str:
         """Get the RETURNING clause of write statement, if any is specified.
 
@@ -341,8 +378,9 @@ class _BaseWriteStatement(BaseSQLStatement):
         try:
             columns = self.fields.fields()
             values = self.fields.values()
-            assert columns and values and len(columns) == len(values), \
-                f"{type(self).__name__}.fields must all have a name and value."
+            assert (
+                columns and values and len(columns) == len(values)
+            ), f"{type(self).__name__}.fields must all have a name and value."
         except AssertionError as err:
             raise TypeError(err)
 
@@ -350,13 +388,16 @@ class _BaseWriteStatement(BaseSQLStatement):
 @dataclasses.dataclass
 class Update(_BaseWriteStatement):
     """A simple, single-table SQL UPDATE Statement."""
+
     table: str
     schema: str = None
     filters: FilterList = dataclasses.field(default_factory=FilterList)
     fields: FieldList = dataclasses.field(default_factory=FieldList)
     returns: Field = None
 
-    def build_update(self, style: ParamStyleType = DEFAULT_PARAM_STYLE) -> Tuple[str, ArgList]:
+    def build_update(
+        self, style: ParamStyleType = DEFAULT_PARAM_STYLE
+    ) -> Tuple[str, ArgList]:
         """Build the SQL UPDATE clause.
 
         Parameters
@@ -372,12 +413,14 @@ class Update(_BaseWriteStatement):
         updates = []
         args = ArgList()
         for field in self.fields:
-            stmt, args = Filter(field, prefix='col').to_sql(args, style)
+            stmt, args = Filter(field, prefix="col").to_sql(args, style)
             updates.append(stmt)
-        updates = ',\n  '.join(updates)
+        updates = ",\n  ".join(updates)
         return f"UPDATE\n  {self.table_name}\nSET\n  {updates}", args
 
-    def to_sql(self, style: ParamStyleType = DEFAULT_PARAM_STYLE) -> Tuple[str, Union[List, Dict]]:
+    def to_sql(
+        self, style: ParamStyleType = DEFAULT_PARAM_STYLE
+    ) -> Tuple[str, Union[List, Dict]]:
         """Build the SQL UPDATE clause.
 
         Parameters
@@ -404,13 +447,20 @@ class Insert(_BaseWriteStatement):
     complex to generate (barring JOIN statements). This statement alone is the reason this library was
     implemented, as I'd grown tired of solving the same problem every time an ORM wasn't a viable option.
     """
+
     table: str
     schema: str = None
     fields: FieldList = dataclasses.field(default_factory=FieldList)
     returns: Field = None
 
     @staticmethod
-    def _fields_to_sql(fields: FieldList, style: ParamStyleType, args: ArgList) -> str:
+    def _fields_to_sql(
+        fields: FieldList, style: ParamStyleType, args: ArgList, inject: bool = False
+    ) -> str:
+        # Override DBAPI formatting. This is dangerous, but required for columns on insert statements in some clients.
+        if inject:
+            return f"({', '.join(fields.values())})"
+
         stmnts = []
         for field in fields:
             # convert the enum to str
@@ -423,11 +473,16 @@ class Insert(_BaseWriteStatement):
             # append the parameter to the list
             stmnts.append(fmt)
         # join the parameters
-        stmnts = ',\n  '.join(stmnts)
+        stmnts = ",\n  ".join(stmnts)
         # return them as a single, SQL-compliant fragment
         return f"({stmnts})" if stmnts else ""
 
-    def build_insert(self, style: ParamStyleType = DEFAULT_PARAM_STYLE) -> Tuple[str, ArgList]:
+    def build_insert(
+        self,
+        style: ParamStyleType = DEFAULT_PARAM_STYLE,
+        *,
+        inject_columns: bool = False,
+    ) -> Tuple[str, ArgList]:
         """Build a SQL INSERT statement.
 
         We create two new :class:`FieldList` - one for column declaration and one for values declaration.
@@ -440,6 +495,9 @@ class Insert(_BaseWriteStatement):
         --------
         style : defaults :class:`NumParamStyle.NUM`
             The DBAPI 2.0 param-style.
+        inject_columns: defaults False
+            Inject the column names directly, rather than rely on DBAPI formatting.
+            This is necessary for some client, such as ``asyncpg``.
 
         Returns
         -----
@@ -448,36 +506,45 @@ class Insert(_BaseWriteStatement):
         """
         columns = FieldList([Field(f"col{x.name}", x.name) for x in self.fields])
         values = FieldList([Field(f"val{x.name}", x.value) for x in self.fields])
-        args = ArgList(columns + values)
-        insert_sql = self._fields_to_sql(columns, style, args)
+        args = ArgList(values) if inject_columns else ArgList(columns + values)
+        insert_sql = self._fields_to_sql(columns, style, args, inject=inject_columns)
         values_sql = self._fields_to_sql(values, style, args)
         returning = self.get_returning()
         return (
-            f"INSERT INTO\n  {self.table_name} {insert_sql}\n"
-            f"VALUES\n  {values_sql}\n"
-            f"{returning}"
-        ), args
+            (
+                f"INSERT INTO\n  {self.table_name} {insert_sql}\n"
+                f"VALUES\n  {values_sql}\n"
+                f"{returning}"
+            ),
+            args,
+        )
 
-    def to_sql(self, style: ParamStyleType = DEFAULT_PARAM_STYLE) -> Tuple[str, Union[List, Dict]]:
+    def to_sql(
+        self, style: ParamStyleType = DEFAULT_PARAM_STYLE, inject_columns: bool = False
+    ) -> Tuple[str, Union[List, Dict]]:
         """Build the SQL INSERT statement and format the list of arguments to pass to the DB client.
 
         Parameters
         --------
         style : defaults :class:`NumParamStyle.NUM`
             The DBAPI 2.0 param-style.
+        inject_columns: defaults False
+            Inject the column names directly, rather than rely on DBAPI formatting.
+            This is necessary for some client, such as ``asyncpg``.
 
         Returns
         -----
         The generated SQL INSERT statement
         The arguments to pass to the DB client for secure formatting.
         """
-        query, args = self.build_insert(style)
+        query, args = self.build_insert(style, inject_columns=inject_columns)
         return query, args.for_sql(style)
 
 
 @dataclasses.dataclass
 class Delete(BaseSQLStatement):
     """A simple, single-table SQL DELETE Statement."""
+
     table: str
     schema: str = None
     filters: FilterList = dataclasses.field(default_factory=FilterList)
@@ -486,7 +553,9 @@ class Delete(BaseSQLStatement):
     def get_returning(self) -> str:
         return f"RETURNING {self.returns.for_fetch()}" if self.returns else ""
 
-    def to_sql(self, style: ParamStyleType = DEFAULT_PARAM_STYLE) -> Tuple[str, Union[List, Dict]]:
+    def to_sql(
+        self, style: ParamStyleType = DEFAULT_PARAM_STYLE
+    ) -> Tuple[str, Union[List, Dict]]:
         """Generate a SQL DELETE statement.
 
         Parameters
@@ -501,11 +570,14 @@ class Delete(BaseSQLStatement):
         """
         where, args = self.filters.to_sql(style=style)
         returning = self.get_returning()
-        return f"DELETE FROM\n  {self.table_name}\n{where}\n{returning}", args.for_sql(style)
+        return (
+            f"DELETE FROM\n  {self.table_name}\n{where}\n{returning}",
+            args.for_sql(style),
+        )
 
 
 FieldDataType = NewType(
-    'FieldDataType', Union[Dict, Collection[Tuple[Hashable, Any]], NamedTuple, Type]
+    "FieldDataType", Union[Dict, Collection[Tuple[Hashable, Any]], NamedTuple, Type]
 )
 
 
@@ -526,7 +598,8 @@ def data_to_fields(data: FieldDataType, exclude: Any = Nothing) -> FieldList:
         elif isnamedtuple(data):
             data = dict_factory(data._asdict())
         elif isinstance(data, Mapping) or (
-            isinstance(data, Collection) and all((isinstance(x, Tuple) and len(x) == 2) for x in data)
+            isinstance(data, Collection)
+            and all((isinstance(x, Tuple) and len(x) == 2) for x in data)
         ):
             data = dict_factory(data)
         if isinstance(data, dict):
